//@flow
import _ from 'lodash';
import q from 'q';
/* eslint-disable-next-line */
const gsapMod = import(/* webpackChunkName: "gsap" */ 'gsap'); // jshint ignore:line

/* delay a bit initial gsap loading, releases the pressure on animation with content loading */
var gsap = q.delay(250).then(() => gsapMod);

/* get the very last argument as a callback, manages an optional parameters */
function getCallback(args) {
  var done = _.last(args);
  return _.isFunction(done) ? done : _.noop;
}

function getStyle(elt) {
  return window.getComputedStyle(elt, null);
}

function getDom(elt /*: any */) {
  if (_.has(elt, 'elm')) {
    /* VNode */
    return elt.elm;
  }
  else if (_.has(elt, '$el')) {
    /* VueComponent */
    return elt.$el;
  }
  return elt;
}

function getRandom(max, min) {
  return Math.floor((Math.random() * (1 + max - min)) + min);
}

/*
 * This is used to chain animations on a component
 */
function attach(gsap, elt, anim) {
  if (!elt.anim || !elt.anim.isPending()) {
    elt.anim = anim();
  }
  else {
    elt.anim = elt.anim.then(anim);
  }
}

function gsapHook(anim /*: (gsap: any, tl: any, elt: any, options: ?{}) => any */) {
  return function(elt /*: any */, options /*: ?{} */, done /*: ?(() => any) */) {
    done = getCallback(arguments); // eslint-disable-line
    elt = getDom(elt);

    gsap.then((gsap) => {
      var def = q.defer();
      elt = getDom(elt);
      const tl = new gsap.TimelineMax({
        paused: true,
        onComplete: () => {
          def.resolve();
          if (done) { done(); }
        }
      });

      tl.add(anim.bind(null, gsap, tl, elt, options));
      attach(gsap, elt, () => {
        tl.play();
        return def.promise;
      });
    })
    .catch((err) => {
      console.log('animation failure', err);
      if (done) { done(); }
    });
  };
}

gsap.then((gsap) => (gsap.CSSPlugin.defaultTransformPerspective = 500));

var BaseAnimation = {
  default: 'fade',
  'setOpacity': function(elt /*: any */, opacity /*: number */) { /* not an animation, just an helper */
    gsap.then((gsap) => {
      elt = getDom(elt);
      gsap.TweenMax.set(elt, { opacity });
    });
  },
  'getDom': getDom,
  'randomTranslateIn': gsapHook(function(gsap, tl, elt) {
    tl.from(elt, 1.2, {
      opacity: 0,
      x: getRandom(300, -300),
      y: getRandom(300, -300),
      ease: gsap.Bounce.easeOut
    });
  }),
  flip: {
    in: gsapHook(function(gsap, tl, elt) {
      tl.fromTo(elt, 0.5,
        { opacity: 0, rotationX: -90 },
        { opacity: 1, rotationX: 0, ease: gsap.Bounce.easeOut });
    }),
    out: gsapHook(function(gsap, tl, elt) {
      tl.to(elt, 0.5, { opacity: 0, rotationX: -90, ease: gsap.Bounce.easeOut });
    })
  },
  rot90: {
    in: gsapHook(function(gsap, tl, elt) {
      tl.to(elt, 0.3, { rotation: 90, ease: gsap.Power2.easeOut });
    }),
    out: gsapHook(function(gsap, tl, elt) {
      tl.to(elt, 0.3, { rotation: 0, ease: gsap.Power2.easeOut });
    })
  },
  fade: {
    in: gsapHook(function(gsap, tl, elt, options) {
      tl.fromTo(elt, _.get(options, 'duration') || 0.3,
        { opacity: _.get(elt.style, 'opacity', 0) || 0 },
        { opacity: 1, ease: gsap.Power2.easeOut });
    }),
    out: gsapHook(function(gsap, tl, elt, options) {
      tl.to(elt, _.get(options, 'duration') || 0.3,
        { opacity: 0, ease: gsap.Power2.easeOut });
    })
  },
  pop: {
    in: gsapHook(function(gsap, tl, elt) {
      gsap.TweenMax.set(elt, {
        visibility: 'hidden',
        attr: { 'data-anim': 'gsap' }
      });
      tl.addLabel('start');
      tl.from(elt, 0.3, {
        'padding-top': 0, 'padding-bottom': 0, height: 0, scale: 0.001,
        ease: gsap.Back.easeOut
      });
      tl.set(elt, { visibility: 'visible' }, 'start');
      tl.set(elt, { clearProps: 'padding-top,padding-bottom,height,scale' });
    }),
    out: gsapHook(function(gsap, tl, elt) {
      tl.to(elt, 0.3, {
        'padding-top': 0, 'padding-bottom': 0, height: 0, scale: 0.001,
        ease: gsap.Back.easeIn
      });
      tl.set(elt, {
        clearProps: 'padding-top,padding-bottom,height,scale',
        visibility: 'hidden'
      });
    })
  },
  height: {
    in: gsapHook(function(gsap, tl, elt) {
      gsap.TweenMax.set(elt, {
        visibility: 'hidden',
        attr: { 'data-anim': 'gsap' }
      });
      tl.addLabel('start');
      tl.from(elt, 0.3, {
        'padding-top': 0, 'padding-bottom': 0, height: 0,
        ease: gsap.Back.easeOut
      });
      tl.set(elt, { visibility: 'visible' }, 'start');
      tl.set(elt, { clearProps: 'padding-top,padding-bottom,height' });
    }),
    out: gsapHook(function(gsap, tl, elt) {
      tl.to(elt, 0.3, {
        'padding-top': 0, 'padding-bottom': 0, height: 0,
        ease: gsap.Back.easeIn
      });
      tl.set(elt, {
        clearProps: 'padding-top,padding-bottom,height',
        visibility: 'hidden'
      });
    })
  },
  width: {
    in: gsapHook(function(gsap, tl, elt) {
      gsap.TweenMax.set(elt, {
        visibility: 'hidden',
        attr: { 'data-anim': 'gsap' }
      });
      tl.addLabel('start');
      tl.from(elt, 0.3, {
        'padding-left': 0, 'padding-right': 0, width: 0,
        ease: gsap.Back.easeOut
      });
      tl.set(elt, { visibility: 'visible' }, 'start');
      tl.set(elt, { clearProps: 'padding-left,padding-right,width' });
    }),
    out: gsapHook(function(gsap, tl, elt) {
      tl.to(elt, 0.3, {
        'padding-left': 0, 'padding-right': 0, width: 0,
        ease: gsap.Back.easeIn
      });
      tl.set(elt, {
        clearProps: 'padding-left,padding-right,width',
        visibility: 'hidden'
      });
    })
  },
  page: {
    in: gsapHook(function(gsap, tl, elt) {
      var style = getStyle(elt);
      tl.set(elt, {
        position: 'absolute', width: style.width, height: style.height,
        y: window.innerHeight,
        scale: 0.8
      })
      .to(elt, 0.5, { y: 0, ease: gsap.Power4.easeOut })
      .to(elt, 1, { scale: 1, ease: gsap.Power4.easeOut })
      .set(elt, { clearProps: 'position,width,height' });
    }),
    out: gsapHook(function(gsap, tl, elt) {
      var style = getStyle(elt);
      tl.to(elt, 1, {
        position: 'absolute', width: style.width, height: style.height,
        y: window.innerHeight,
        ease: gsap.Power4.easeOut
      })
      .set(elt, { clearProps: 'position,width,height' });
    })
  }
};

export default BaseAnimation;
